package tools.descartes.dql.connector.simQPN;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;

import javax.xml.XMLConstants;

import org.apache.log4j.Logger;
import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.Element;
import org.dom4j.Namespace;
import org.dom4j.QName;
import org.dom4j.io.OutputFormat;
import org.dom4j.io.SAXReader;
import org.dom4j.io.XMLWriter;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IPath;

import tools.descartes.dql.core.engine.util.DQLLogger;
import de.tud.cs.qpe.model.NetHelper;

public class NetUtil {

	private static final Logger log = DQLLogger.getLogger(NetUtil.class.getName());
	
	public static Element loadXmlFromFile(String path) {
		//check for local path
		if (path.matches("(\\\\|\\/).*")) {
			final IPath workspaceLocation = ResourcesPlugin.getWorkspace()
					.getRoot().getLocation();
			final IPath workspaceModelLocation = workspaceLocation
					.append(path);
			final File file = workspaceModelLocation.toFile();
			path = FileUtil.getURL(file);
		}
		
		SAXReader xmlReader = new SAXReader();
		Document inputDoc = null;
		try {
			inputDoc = xmlReader.read(path);
		} catch (DocumentException e2) {
			log.error(e2.getMessage());
			throw new DQLException("Could not find the specified file or it is not a QPN net! \nPath:");
		}
		return inputDoc.getRootElement();
	}

	public static String getNameOfUnderlyingQueue(String placeName, Element net) {
		Element p = NetUtil.getPlaceByName(placeName, net);
		String queueId = p.attributeValue(new QName("queue-ref"));
		if (queueId == null)
			throw new DQLException("That is not a queueing Place");
		String queueName = NetUtil.getNameOfElement(NetHelper.getQueueById(net, queueId));
		return queueName;
	}
	
	private static String getUnderlyingQueueName(Element p, Element net) {
		String qId = p.attributeValue(new QName("queue-ref"));
		if (qId == null)
			return null;
		Element queue = NetHelper.getQueueById(net, qId);
		return getNameOfElement(queue);
	}

	public static String getNameOfElement(Element e) {
		return e.attributeValue(new org.dom4j.QName("name"));
	}

	public static String getIdOfElement(Element e) {
		return e.attributeValue(new org.dom4j.QName("id"));
	}

	public static String getTypeOfPlace(Element place) {
		String type = place.attributeValue(new org.dom4j.QName("type"));
		if (type == null)
			type = place.attributeValue(new org.dom4j.QName("type",
					new Namespace("xsi", XMLConstants.W3C_XML_SCHEMA_INSTANCE_NS_URI)));
		if (type == null)
			return "queue";
		return type;
	}
	
	public static String getIdOfPlaceByName(String name, Element net) {
		return getIdOfElement(getPlaceByName(name, net));
	}
	
	public static String removeQueueTag(String name) {
		return name.substring(6);
	}
	
	public static Element getProbeByName(String name, Element net) {
		for (Element e: NetHelper.listProbes(net))
			if (getNameOfElement(e).equals(name))
				return e;
		return null;
	}
	
	public static Element getPlaceByName(String name, Element net) {
		for (Element e: NetHelper.listAllPlaces(net))
			if (getNameOfElement(e).equals(name))
				return e;
		return null;
	}
	
	public static String getReferencedColorId(Element e) {
		return e.attributeValue(new org.dom4j.QName("color-id"));
	}
	
	public static void save(File path, Element doc) {
        XMLWriter writer = null;
        try {
            OutputFormat format = OutputFormat.createPrettyPrint();
            writer = new XMLWriter(new FileOutputStream(path), format);
            writer.write(doc);
        } catch (IOException e) {
            log.error(e.getMessage());
        } finally {
            if (writer != null) {
                try {
                    writer.close();
                } catch (IOException e) {
                    log.error(e.getMessage());
                }
            }
        }
    }

	public static String getIdOfPlaceWithQueue(String queueName, Element net) {
		for (Element place: NetHelper.listPlaces(net))
			if (getUnderlyingQueueName(place, net) != null &&
				getUnderlyingQueueName(place, net).equals(queueName))
				return getIdOfElement(place);
		return null;
	}

	public static String getIdOfProbeByName(String identifier, Element net) {
		return getIdOfElement(getProbeByName(identifier, net));
	}

	public static Element getProbeById(Element net, String id) {
		for (Element e: NetHelper.listProbes(net))
			if (getIdOfElement(e).equals(id))
				return e;
		return null;
	}
}
