package tools.descartes.dql.connector.simQPN;

import java.util.Collection;
import java.util.LinkedList;
import java.util.List;
import java.util.regex.Pattern;

import org.apache.log4j.Logger;

import tools.descartes.dql.core.engine.util.DQLLogger;
import de.tud.cs.qpe.editors.query.model.Metric;

public class PerformanceMetricsHelper {
	private static final Logger log = DQLLogger.getLogger(PerformanceMetricsHelper.class.getName());
	private static final String colorPlaceHolder = "$color$";
	private static final String queuePlaceHolder = "$queue$";
	
	public static PerformanceMetric parsePerformanceMetric(String metricName, String placeType) {
		List<PerformanceMetric> allMetrics = null;
		if (placeType.equals("queue"))
			allMetrics = getPerformanceMetricsForQueue();
		if (placeType.equals("queueing-place"))
			allMetrics = getPerformanceMetricsForQueueingPlaceWithPlaceHolder();
		if (placeType.equals("ordinary-place"))
			allMetrics = getPerformanceMetricsForPlaceWithPlaceHolder();
		if (placeType.equals("probe"))
			allMetrics = getPerformanceMetricsForProbeWithPlaceHolder();
		for (PerformanceMetric pm: allMetrics) {
			//Check if it is the right metric
			String regex = pm.getDQLName();
			regex = regex.replace(colorPlaceHolder, "([^_]*)");
//			regex = regex.replace(queuePlaceHolder, "([^_]*)");
			if (metricName.matches(regex)) {
//				pm.setQueueName(extractQueueName(metricName, pm.getDQLName()));
				pm.setColorName(extractColorName(metricName, pm.getDQLName()));
				return pm;
			}
		}
		log.error("There is no metric with that name: " + metricName);
		throw new DQLException("There is no metric with the name " + metricName);
	}
	
//	private static String extractQueueName(String inputMetricName, String metricName) {
//		if (!metricName.contains(queuePlaceHolder))
//			return null;
//		String regex = metricName.replace(colorPlaceHolder, "[^_]+");
//		regex = regex.replace(queuePlaceHolder, "([^_]+)");
//		Pattern pattern = Pattern.compile(regex);
//		java.util.regex.Matcher m = pattern.matcher(inputMetricName);
//		m.matches();
//		return m.group(1);
//	}
	
	private static List<PerformanceMetric> getPerformanceMetricsForProbeWithPlaceHolder() {
		List<String> colorNames = new LinkedList<String>();
		colorNames.add(colorPlaceHolder);
		return getPerformanceMetricsForProbe(colorNames);
	}

	private static String extractColorName(String inputMetricName, String metricName) {
		if (!metricName.contains(colorPlaceHolder))
			return null;
		String regex = metricName.replace(queuePlaceHolder, "[^_]+");
		regex = regex.replace(colorPlaceHolder, "([^_]+)");
		Pattern pattern = Pattern.compile(regex);
		java.util.regex.Matcher m = pattern.matcher(inputMetricName);
		m.matches();
		return m.group(1);
	}

	public static List<PerformanceMetric> getAllPossiblePerformanceMetricsWithPlaceHolder() {
		List<PerformanceMetric> metrics = getPerformanceMetricsForPlaceWithPlaceHolder();
		for (PerformanceMetric pm: getPerformanceMetricsForQueueingPlaceWithPlaceHolder()) 
			if (!metrics.contains(pm))
				metrics.add(pm);
		return metrics;
	}
	
	public static List<PerformanceMetric> getPerformanceMetricsForProbe(List<String> colorNames) {
		List<PerformanceMetric> metrics = new LinkedList<PerformanceMetric>();
		
		//Level 1
		for (String colorName: colorNames) {
			metrics.add(new PerformanceMetric(Metric.ARRIV_THR_PUT, 1, colorName, false));
			metrics.add(new PerformanceMetric(Metric.DEPT_THR_PUT, 1, colorName, false));
		}
		
		//Level 2
		for (String colorName: colorNames) {
			metrics.add(new PerformanceMetric(Metric.TK_COL_OCP, 2, colorName, false));
			metrics.add(new PerformanceMetric(Metric.MIN_TK_POP, 2, colorName, false));
			metrics.add(new PerformanceMetric(Metric.MAX_TK_POP, 2, colorName, false));
			metrics.add(new PerformanceMetric(Metric.MEAN_TK_POP, 2, colorName, false));
		}
		
		//Level 3
		for (String colorName: colorNames) {
			metrics.add(new PerformanceMetric(Metric.MAX_ST, 3, colorName, false));
			metrics.add(new PerformanceMetric(Metric.MEAN_ST, 3, colorName, false));
			metrics.add(new PerformanceMetric(Metric.ST_DEV_ST, 3, colorName, false));
			metrics.add(new PerformanceMetric(Metric.MIN_ST, 3, colorName, false));
		}
		return metrics;
	}
	
	public static List<PerformanceMetric> getPerformanceMetricsForPlace(List<String> colorNames) {
		List<PerformanceMetric> metrics = new LinkedList<PerformanceMetric>();
		
		//Level 1
		for (String colorName: colorNames) {
			metrics.add(new PerformanceMetric(Metric.ARRIV_THR_PUT, 1, colorName, false));
			metrics.add(new PerformanceMetric(Metric.DEPT_THR_PUT, 1, colorName, false));
		}
		
		//Level 2
		metrics.add(new PerformanceMetric(Metric.TK_OCP, 2, null, false));
		for (String colorName: colorNames) {
			metrics.add(new PerformanceMetric(Metric.TK_COL_OCP, 2, colorName, false));
			metrics.add(new PerformanceMetric(Metric.MIN_TK_POP, 2, colorName, false));
			metrics.add(new PerformanceMetric(Metric.MAX_TK_POP, 2, colorName, false));
			metrics.add(new PerformanceMetric(Metric.MEAN_TK_POP, 2, colorName, false));
		}
		
		//Level 3
		for (String colorName: colorNames) {
			metrics.add(new PerformanceMetric(Metric.MAX_ST, 3, colorName, false));
			metrics.add(new PerformanceMetric(Metric.MEAN_ST, 3, colorName, false));
			metrics.add(new PerformanceMetric(Metric.ST_DEV_ST, 3, colorName, false));
			metrics.add(new PerformanceMetric(Metric.MIN_ST, 3, colorName, false));
		}
		return metrics;
	}
	
	public static List<PerformanceMetric> getPerformanceMetricsForPlaceWithPlaceHolder() {
		List<String> colorNames = new LinkedList<String>();
		colorNames.add(colorPlaceHolder);
		return getPerformanceMetricsForPlace(colorNames);
	}
	
	public static List<PerformanceMetric> getPerformanceMetricsForQueueingPlace(List<String> colorNames, String queueName) {
		List<PerformanceMetric> metrics = new LinkedList<PerformanceMetric>();
		metrics.addAll(getPerformanceMetricsForQueuePlace(colorNames));
		metrics.addAll(getPerformanceMetricsForQueueDepository(colorNames));
		return metrics;
	}
	
	public static List<PerformanceMetric> getPerformanceMetricsForQueueingPlaceWithPlaceHolder() {
		List<String> colorNames = new LinkedList<String>();
		colorNames.add(colorPlaceHolder);
		return getPerformanceMetricsForQueueingPlace(colorNames, queuePlaceHolder);
	}
	
	private static Collection<? extends PerformanceMetric> getPerformanceMetricsForQueueDepository(
			List<String> colorNames) {
		List<PerformanceMetric> metrics = new LinkedList<PerformanceMetric>();
		
		//Level 1
		for (String colorName: colorNames) {
			metrics.add(new PerformanceMetric(Metric.ARRIV_THR_PUT, 1, colorName, false, true));
			metrics.add(new PerformanceMetric(Metric.DEPT_THR_PUT, 1, colorName, false, true));
		}
		
		//Level 2
		metrics.add(new PerformanceMetric(Metric.TK_OCP, 2, null, false, true));
		for (String colorName: colorNames) {
			metrics.add(new PerformanceMetric(Metric.TK_COL_OCP, 2, colorName, false, true));
			metrics.add(new PerformanceMetric(Metric.MIN_TK_POP, 2, colorName, false, true));
			metrics.add(new PerformanceMetric(Metric.MAX_TK_POP, 2, colorName, false, true));
			metrics.add(new PerformanceMetric(Metric.MEAN_TK_POP, 2, colorName, false, true));
		}
		
		//Level 3
		for (String colorName: colorNames) {
			metrics.add(new PerformanceMetric(Metric.MAX_ST, 3, colorName, false, true));
			metrics.add(new PerformanceMetric(Metric.MEAN_ST, 3, colorName, false, true));
			metrics.add(new PerformanceMetric(Metric.ST_DEV_ST, 3, colorName, false, true));
			metrics.add(new PerformanceMetric(Metric.MIN_ST, 3, colorName, false, true));
		}
		return metrics;
	}

	private static Collection<? extends PerformanceMetric> getPerformanceMetricsForQueuePlace(
			List<String> colorNames) {
		List<PerformanceMetric> metrics = new LinkedList<PerformanceMetric>();
		
		//Level 1
		metrics.add(new PerformanceMetric(Metric.QUEUE_UTIL_QP1, 2, null, false));
		
		//Level 2
		for (String colorName: colorNames) {
			metrics.add(new PerformanceMetric(Metric.DEPT_THR_PUT, 2, colorName, false));
			metrics.add(new PerformanceMetric(Metric.TK_COL_OCP, 2, colorName, false));
			metrics.add(new PerformanceMetric(Metric.MIN_TK_POP, 2, colorName, false));
			metrics.add(new PerformanceMetric(Metric.ARRIV_THR_PUT, 2, colorName, false));
			metrics.add(new PerformanceMetric(Metric.MAX_TK_POP, 2, colorName, false));
			metrics.add(new PerformanceMetric(Metric.MEAN_TK_POP, 2, colorName, false));
		}
		
		//Level 3
		for (String colorName: colorNames) {
			metrics.add(new PerformanceMetric(Metric.MAX_ST, 3, colorName, false));
			metrics.add(new PerformanceMetric(Metric.MEAN_ST, 3, colorName, false));
			metrics.add(new PerformanceMetric(Metric.ST_DEV_ST, 3, colorName, false));
			metrics.add(new PerformanceMetric(Metric.MIN_ST, 3, colorName, false));
		}
		return metrics;
	}

	public static List<PerformanceMetric> getPerformanceMetricsForQueue() {
		List<PerformanceMetric> metrics = new LinkedList<PerformanceMetric>();
		
		//Level 1
		metrics.add(new PerformanceMetric(Metric.TOT_ARRIV_THR_PUT, 1, null, true));
		metrics.add(new PerformanceMetric(Metric.TOT_DEPT_THR_PUT, 1, null, true));
		
		//Level 2
		metrics.add(new PerformanceMetric(Metric.QUEUE_UTIL, 2, null, true));
		
		//Level 3
		metrics.add(new PerformanceMetric(Metric.MEAN_ST, 3, null, true));
		
		return metrics;
	}

	public static String getDQLNameForMetric(Metric simqpnMetric) {
		switch (simqpnMetric) {
		case ARRIV_THR_PUT:
			return "ArrivalThroughput";
		case CONFIDENCE_INTERVAL_LENGTH:
			return "ConfidenceIntervalHalfLength";
		case CONFIDENCE_INTERVAL_MEAN:
			return "ConfidenceIntervalMean";
		case DEPT_THR_PUT:
			return "DepartureThroughput";
		case MAX_ST:
			return "MaximumTokenResidenceTime";
		case MAX_TK_POP:
			return "MaximumTokenPopulation";
		case MEAN_ST:
			return "MeanTokenResidenceTime";
		case MEAN_TK_POP:
			return "MeanTokenPopulation";
		case MEAN_TOT_TK_POP:
			return "MeanTotalTokenPopulation";
		case MIN_ST:
			return "MinimumTokenResidenceTime";
		case MIN_TK_POP:
			return "MinimumTokenPopulation";
		case QUEUE_UTIL:
			return "QueueUtilization";
		case QUEUE_UTIL_QP1:
			return "QueueUtilizationDueToThisPlace";
		case ST_DEV_ST:
			return "StandardDeviationTokenResidenceTime";
		case TK_COL_OCP:
			return "TokenColorOccupancy";
		case TK_OCP:
			return "TokenOccupancy";
		case TOT_ARRIV_THR_PUT:
			return "TotalArrivalThroughput";
		case TOT_DEPT_THR_PUT:
			return "TotalDepartureThroughput";
		default:
			return "ERROR";
		}
	}
	
//	private static List<PerformanceMetric> loadPlaceSpecificMetrics() {
//		List<PerformanceMetric> placeSpecificMetrics = new LinkedList<PerformanceMetric>();
//		placeSpecificMetrics.add(new PerformanceMetric(getDQLNameForMetric(Metric.TK_OCP), Metric.TK_OCP, 2));
//		return placeSpecificMetrics;
//	}
//	
//	private static List<PerformanceMetric> loadQueueSpecificMetrics() {
//		List<PerformanceMetric> queueSpescificMetrics = new LinkedList<PerformanceMetric>();
//		queueSpescificMetrics.add(new PerformanceMetric(Metric.QUEUE_UTIL_QP1, 2));
//		queueSpescificMetrics.add(new PerformanceMetric("UnderlyingQueue_MeanTokenResidenceTime",
//				Metric.MEAN_ST, 3));
//		queueSpescificMetrics.add(new PerformanceMetric("UnderlyingQueue_MeanTotalTokenPopulation",
//				Metric.MEAN_TK_POP, 2));
//		queueSpescificMetrics.add(new PerformanceMetric("UnderlyingQueue_QueueUtilization",
//				Metric.QUEUE_UTIL, 3));
//		queueSpescificMetrics.add(new PerformanceMetric("UnderlyingQueue_TotalArrivalThroughput",
//				Metric.TOT_ARRIV_THR_PUT, 1));
//		queueSpescificMetrics.add(new PerformanceMetric("UnderlyingQueue_TotalDepartureThroughput",
//				Metric.TOT_DEPT_THR_PUT, 1));
//		return queueSpescificMetrics;
//	}
//	
//	private static List<PerformanceMetric> loadPlaceColorSpecificNames(String colorName) {
//		List<PerformanceMetric> placeColorSpescificMetrics = new LinkedList<PerformanceMetric>();
//		placeColorSpescificMetrics.add(new PerformanceMetric(colorName + "_" + getDQLNameForMetric(Metric.ARRIV_THR_PUT), 
//				Metric.ARRIV_THR_PUT, 1, colorName));
//		placeColorSpescificMetrics.add(new PerformanceMetric(colorName + "_" + getDQLNameForMetric(Metric.CONFIDENCE_INTERVAL_LENGTH),
//				Metric.CONFIDENCE_INTERVAL_LENGTH, 3, colorName));
//		placeColorSpescificMetrics.add(new PerformanceMetric(colorName + "_" + getDQLNameForMetric(Metric.CONFIDENCE_INTERVAL_MEAN),
//				Metric.CONFIDENCE_INTERVAL_MEAN, 3, colorName));
//		placeColorSpescificMetrics.add(new PerformanceMetric(colorName + "_" + getDQLNameForMetric(Metric.DEPT_THR_PUT),
//				Metric.DEPT_THR_PUT, 1, colorName));
//		placeColorSpescificMetrics.add(new PerformanceMetric(colorName + "_" + getDQLNameForMetric(Metric.MAX_TK_POP),
//				Metric.MAX_TK_POP, 2, colorName));
//		placeColorSpescificMetrics.add(new PerformanceMetric(colorName + "_" + getDQLNameForMetric(Metric.MAX_ST),
//				Metric.MAX_ST, 1, colorName));
//		placeColorSpescificMetrics.add(new PerformanceMetric(colorName + "_" + getDQLNameForMetric(Metric.MEAN_TK_POP),
//				Metric.MEAN_TK_POP, 2, colorName));
//		placeColorSpescificMetrics.add(new PerformanceMetric(colorName + "_" + getDQLNameForMetric(Metric.MEAN_ST),
//				Metric.MEAN_ST, 3, colorName));
//		placeColorSpescificMetrics.add(new PerformanceMetric(colorName + "_" + getDQLNameForMetric(Metric.MIN_TK_POP),
//				Metric.MIN_TK_POP, 2, colorName));
//		placeColorSpescificMetrics.add(new PerformanceMetric(colorName + "_" + getDQLNameForMetric(Metric.MIN_ST),
//				Metric.MIN_ST, 3, colorName));
//		placeColorSpescificMetrics.add(new PerformanceMetric(colorName + "_" + getDQLNameForMetric(Metric.ST_DEV_ST),
//				Metric.ST_DEV_ST, 3, colorName));
//		placeColorSpescificMetrics.add(new PerformanceMetric(colorName + "_" + getDQLNameForMetric(Metric.TK_COL_OCP),
//				Metric.TK_COL_OCP, 3, colorName));
//		return placeColorSpescificMetrics;
//	}
//	
//	private static List<PerformanceMetric> loadPlaceColorSpecificNames(List<String> colorNames) {
//		List<PerformanceMetric> placeColorSpescificMetrics = new LinkedList<PerformanceMetric>();
//		for (String colorName : colorNames)
//			placeColorSpescificMetrics.addAll(loadPlaceColorSpecificNames(colorName));
//		return placeColorSpescificMetrics;
//	}
//	
//	public static List<PerformanceMetric> getPlaceMetricsWithPlaceholders() {
//		List<PerformanceMetric> placeMetrics = new LinkedList<PerformanceMetric>();
//		placeMetrics.addAll(loadPlaceSpecificMetrics());
//		placeMetrics.addAll(loadPlaceColorSpecificNames(colorPlaceHolder));
//		return placeMetrics;
//	}
//	
//	public static List<PerformanceMetric> getPlaceMetrics(List<String> colorNames) {
//		List<PerformanceMetric> placeMetrics = new LinkedList<PerformanceMetric>();
//		placeMetrics.addAll(loadPlaceSpecificMetrics());
//		placeMetrics.addAll(loadPlaceColorSpecificNames(colorNames));
//		return placeMetrics;
//	}
//	
//	public static List<PerformanceMetric> getAllMetricsWithPlaceHolders() {
//		return getQueueMetricsWithPlaceHolders();
//	}
//	
//	public static List<PerformanceMetric> getQueueMetricsWithPlaceHolders() {
//		List<PerformanceMetric> queueMetrics = new LinkedList<PerformanceMetric>();
//		queueMetrics.addAll(loadQueueSpecificMetrics());
//		List<String> colorNames = new LinkedList<String>();
//		colorNames.add(colorPlaceHolder);
//		queueMetrics.addAll(getPlaceMetrics(colorNames));
//		
//		//Depository has the same metrics as a normal place
//		for (PerformanceMetric pm : getPlaceMetricsWithPlaceholders())
//			queueMetrics.add(new PerformanceMetric("depository_" + pm.getDQLName(), pm.getSimQPNMetric(), pm.getStatLevel()));
//		return queueMetrics;
//	}
//	
//	public static PerformanceMetric parsePerformanceMetric(String metricName) {
//		List<PerformanceMetric> performanceMetrics = getAllMetricsWithPlaceHolders();
//		for (PerformanceMetric pm : performanceMetrics) {
//			if (pm.getDQLName().contains(colorPlaceHolder)) {
//				String[] splits = pm.getDQLName().split(Pattern.quote(colorPlaceHolder), -1);
//				if (splits.length != 2) {
//					throw new IllegalArgumentException("no '$color$' specified");
//				}
//				Pattern pattern = Pattern.compile(
//						splits[0] +  "([^_]*)" + splits[1]);
//				Matcher m = pattern.matcher(metricName);
//				if (m.matches())
//					return new PerformanceMetric(metricName, pm.getSimQPNMetric(), pm.getStatLevel(), m.group(1));
//			} else {
//				if (pm.getDQLName().matches(metricName))
//					return new PerformanceMetric(metricName, pm.getSimQPNMetric(), pm.getStatLevel());
//			}
//					
//		}
//		log.error("There is no metric with that name: " + metricName);
//		throw new DQLException("There is no metric with the name " + metricName);
//	}
//
//	public static List<PerformanceMetric> getQueueMetrics(List<String> colorNames) {
//		List<PerformanceMetric> queueMetrics = new LinkedList<PerformanceMetric>();
//		queueMetrics.addAll(loadQueueSpecificMetrics());
//		queueMetrics.addAll(loadPlaceColorSpecificNames(colorNames));
//			//Depository has the same metrics as a normal place
//		for (PerformanceMetric pm : getPlaceMetrics(colorNames))
//			queueMetrics.add(new PerformanceMetric("depository_" + pm.getDQLName(), pm.getSimQPNMetric(), 
//					pm.getStatLevel(), pm.getColorName()));
//		return queueMetrics;
//	}
//
//	public static String getDQLNameForMetric(Metric simqpnMetric) {
//		switch (simqpnMetric) {
//		case ARRIV_THR_PUT:
//			return "ArrivalThroughput";
//		case CONFIDENCE_INTERVAL_LENGTH:
//			return "ConfidenceIntervalHalfLength";
//		case CONFIDENCE_INTERVAL_MEAN:
//			return "ConfidenceIntervalMean";
//		case DEPT_THR_PUT:
//			return "DepartureThroughput";
//		case MAX_ST:
//			return "MaximumTokenResidenceTime";
//		case MAX_TK_POP:
//			return "MaximumTokenPopulation";
//		case MEAN_ST:
//			return "MeanTokenResidenceTime";
//		case MEAN_TK_POP:
//			return "MeanTokenPopulation";
//		case MEAN_TOT_TK_POP:
//			return "MeanTotalTokenPopulation";
//		case MIN_ST:
//			return "MinimumTokenResidenceTime";
//		case MIN_TK_POP:
//			return "MinimumTokenPopulation";
//		case QUEUE_UTIL:
//			return "QueueUtilization";
//		case QUEUE_UTIL_QP1:
//			return "QueueUtilizationDueToThisPlace";
//		case ST_DEV_ST:
//			return "StandardDeviationTokenResidenceTime";
//		case TK_COL_OCP:
//			return "TokenColorOccupancy";
//		case TK_OCP:
//			return "TokenOccupancy";
//		case TOT_ARRIV_THR_PUT:
//			return "TotalArrivalThroughput";
//		case TOT_DEPT_THR_PUT:
//			return "TotalDepartureThroughput";
//		default:
//			return "ERROR";
//		}
//	}
//
//	public static MetricValue getMetricValue(SimulationResults sr, String placeName,
//			PerformanceMetric performanceMetric, Element net) {
//		if (performanceMetric.getDQLName().matches("UnderlyingQueue_.*"))
//			return getQueueMetricValue(sr, performanceMetric, net, placeName);
//		if (performanceMetric.getDQLName().matches("depository_.*"))
//			return getDepositoryMetricValue(sr, performanceMetric, net, placeName);
//		if (performanceMetric.hasColor()) {
//			return getColorMetricValue(sr, performanceMetric, net, placeName);
//		}
//		Map<String, Place> nameToPlace = new HashMap<String, Place>();
//		for (Place p : sr.getPlaces())
//			nameToPlace.put(p.getName(), p);
//		return sr.getMetricValue(nameToPlace.get(placeName), performanceMetric.getSimQPNMetric());
//	}
//
//	private static MetricValue getColorMetricValue(SimulationResults sr,
//			PerformanceMetric performanceMetric, Element net, String placeName) {
//		Map<String, Color> nameToColor = new HashMap<String, Color>();
//		Map<String, Place> nameToPlace = new HashMap<String, Place>();
//		for (Place p : sr.getPlaces())
//			nameToPlace.put(p.getName(), p);
//		for (Color c: sr.getColors())
//			nameToColor.put(c.getName(), c);
//		return sr.getMetricValue(nameToPlace.get(placeName),
//				nameToColor.get(performanceMetric.getColorName()),
//				performanceMetric.getSimQPNMetric());
//	}
//
//	private static MetricValue getDepositoryMetricValue(SimulationResults sr,
//			PerformanceMetric performanceMetric, Element net, String placeName) {
//		Map<String, Place> nameToPlace = new HashMap<String, Place>();
//		for (Place p : sr.getPlaces())
//			if (p.getType().equals("qplace:depository"))
//				nameToPlace.put(p.getName(), p);
//		if (performanceMetric.hasColor()) {
//			Map<String, Color> nameToColor = new HashMap<String, Color>();
//			for (Color c: sr.getColors())
//				nameToColor.put(c.getName(), c);
//			return sr.getMetricValue(nameToPlace.get(placeName), 
//					nameToColor.get(performanceMetric.getColorName()),
//					performanceMetric.getSimQPNMetric());
//		}
//		return sr.getMetricValue(nameToPlace.get(placeName),
//				performanceMetric.getSimQPNMetric());
//	}
//
//	private static MetricValue getQueueMetricValue(SimulationResults sr,
//			PerformanceMetric performanceMetric, Element net, String placeName) {
//		String queueName = NetUtil.getNameOfUnderlyingQueue(placeName, net);
//		Map<String, Place> nameToPlace = new HashMap<String, Place>();
//		for (Place p : sr.getPlaces())
//			if (p.getType().equals("queue"))
//				nameToPlace.put(p.getName(), p);
//		return sr.getMetricValue(nameToPlace.get(queueName), performanceMetric.getSimQPNMetric());
//	}
//	
}
