package tools.descartes.dql.connector.simQPN;

import static org.junit.Assert.*;

import org.junit.Test;

import tools.descartes.dql.connector.simQPN.testUtils.TestUtil;
import tools.descartes.dql.models.mapping.mapping.EntityMapping;
import tools.descartes.dql.models.mapping.mapping.MappingFactory;
import tools.descartes.dql.models.mapping.mapping.impl.MappingFactoryImpl;

/*
 * This test checks if the connectors methods return the correct EntityMappings.
 * To do this, the expected results are imported from a file and compared to the
 * String version of the EntityMapping.
 * To override the expected results you have to:
 * 1. Uncomment the TestUtil.saveToFile line
 * 2. Run the test (always returns success)
 * 3. Move the files from target/classes to the testfiles folder
 * 4. Comment the TestUtil.saveToFile lines out again
 * 5. Tests now check against the new expected results
 */

public class ModelStructureQueryConnectorImplTest {
	
	@Test
	public void requestEntitiesTest() {
		String modelName = "ispass03.qpe";
		String resultName = "expectedEntities_ispass03.txt";
		String modelPath = FileUtil.getPath(modelName);
		String resultPath = FileUtil.getPath(resultName);
		
		//Check if entity requests are enabled
		ModelStructureQueryConnectorImpl msqc = new ModelStructureQueryConnectorImpl();
		if (!msqc.supportsEntityRequests())
			fail("Entity requests are not supported!");
		
		//Build request
		MappingFactory mappingFactory = MappingFactoryImpl.init();
		EntityMapping request = mappingFactory.createEntityMapping();
		request.setModelLocation(modelPath);
		
		//Run request
		EntityMapping answer = msqc.requestEntities(request);

//		TestUtil.saveToFile(resultPath, TestUtil.entityMapptingToString(answer));
		
		//check for expected answers
		if (!TestUtil.contentCompare(TestUtil.loadFromFile(resultPath),
				TestUtil.entityMapptingToString(answer))) {
			assertEquals(TestUtil.loadFromFile(resultPath),
					TestUtil.entityMapptingToString(answer));
		}
	}

	@Test
	public void requestMetricsTest() {
		String modelName = "ispass03.qpe";
		String resultName = "expectedMetrics_ispass03.txt";
		String modelPath = FileUtil.getPath(modelName);
		String resultPath = FileUtil.getPath(resultName);
		
		//Check if entity requests are enabled
		ModelStructureQueryConnectorImpl msqc = new ModelStructureQueryConnectorImpl();
		if (!msqc.supportsProbeRequests())
			fail("Metric requests are not supported!");
		
		//Build request
		MappingFactory mappingFactory = MappingFactoryImpl.init();
		EntityMapping request = mappingFactory.createEntityMapping();
		request.setModelLocation(modelPath);
		
		//Run request
		request = msqc.requestEntities(request);
		EntityMapping answer = msqc.requestProbes(request);
		
//		TestUtil.saveToFile(resultPath, TestUtil.entityMapptingToString(answer));
		
		if (!TestUtil.contentCompare(TestUtil.loadFromFile(resultPath),
				TestUtil.entityMapptingToString(answer))) {
			assertEquals(TestUtil.loadFromFile(resultPath),
					TestUtil.entityMapptingToString(answer));
		}
		
	}
	

	@Test
	public void requestDoFTest() { 
		String modelName = "ispass03.qpe";
		String resultName = "expectedDoF_ispass03.txt";
		String modelPath = FileUtil.getPath(modelName);
		String resultPath = FileUtil.getPath(resultName);
	    
		//Check if entity requests are enabled
		ModelStructureQueryConnectorImpl msqc = new ModelStructureQueryConnectorImpl();
		if (!msqc.supportsDoFRequests())
			fail("DoF requests are not supported!");
		
		//get EntityMapping from ModelStructureQueryConnector
		MappingFactory mappingFactory = MappingFactoryImpl.init();
		EntityMapping request = mappingFactory.createEntityMapping();
		request.setModelLocation(modelPath);
		request = msqc.requestEntities(request);
		request = msqc.requestProbes(request);
		EntityMapping answer = msqc.requestDoF(request);
		
//		TestUtil.saveToFile(resultPath, TestUtil.entityMapptingToString(answer));

		if (!TestUtil.contentCompare(TestUtil.loadFromFile(resultPath),
				TestUtil.entityMapptingToString(answer))) {
			assertEquals(TestUtil.loadFromFile(resultPath),
					TestUtil.entityMapptingToString(answer));
		}
	}

}
