package tools.descartes.dql.connector.simQPN;

import static org.junit.Assert.*;

import java.util.Arrays;
import java.util.Collection;
import java.util.Properties;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;
import org.junit.runners.Parameterized.Parameter;
import org.junit.runners.Parameterized.Parameters;

import tools.descartes.dql.connector.simQPN.testUtils.TestUtil;
import tools.descartes.dql.models.mapping.domain.DoF;
import tools.descartes.dql.models.mapping.mapping.EntityMapping;
import tools.descartes.dql.models.mapping.mapping.MappingFactory;
import tools.descartes.dql.models.mapping.mapping.impl.MappingFactoryImpl;

@RunWith(Parameterized.class)
public class XMLModificationForDofTest {
    @Parameters
    public static Collection<Object[]> data() {
        return Arrays.asList(new Object[][] {     
        		{"expectedModifiedXML1.qpe", "ispass03.qpe", "WLS-Thread-Pool_t", 3},
           });
    }
    
    @Parameter(value = 0)
	public String resultName;
	@Parameter(value = 1)
	public String modelName;
	@Parameter(value = 2)
	public String dofIdentifier;
	@Parameter(value = 3)
	public int dofValue;
	
	@Test
	public void xmlModificationForDofTest() {
		PerformanceMetricsQueryConnectorImpl pmqc = new PerformanceMetricsQueryConnectorImpl();
		EntityMapping query = createQuery(dofIdentifier, dofValue);
		EntityMapping result = pmqc.requestDoFMetrics(query).get(0);
		String modifiedXML = removeChangingIds(TestUtil.loadFromFile(result.getDoFModelLocation()));
		String expectedXML = removeChangingIds(TestUtil.loadFromFile(FileUtil.getPath(resultName)));
		assertEquals(modifiedXML, expectedXML);
	}
		
	private String removeChangingIds(String s) {
		return s.replaceAll("event-manager-id=\"_[0-9]*\"", "")
				.replaceAll("DQLConfiguration\" id=\"_[0-9]*\"", "");
	}
	
	private static EntityMapping createQuery(String dofName, int dofValue) {
		MappingFactory mappingFactory = MappingFactoryImpl.init();
		EntityMapping entity = mappingFactory.createEntityMapping();
		
		ModelStructureQueryConnectorImpl msqc = new ModelStructureQueryConnectorImpl();
		entity = msqc.requestEntities(entity);
		entity = msqc.requestProbes(entity);
		DoF dof = EntityMappingUtil.createDof("test");
		dof.setProperties(new Properties());
		dof.getProperties().setProperty("value", "" + dofValue);
		dof.setIdentifier(dofName);
		entity.getDoF().add(dof);
	return entity;
	}


}
