package extension;

import java.util.Properties;

import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import de.tud.cs.qpe.model.NetHelper;
import de.tud.cs.qpe.model.PlaceHelper;
import tools.descartes.dql.connector.simQPNV2.DQLException;
import tools.descartes.dql.models.mapping.domain.DoF;
import tools.descartes.dql.models.mapping.domain.impl.DoFImpl;

public class ExtendedDoFInitialTokenPopulation extends DoFImpl implements ExtendedDoF{
	private int[] values;
	private String placeName;
	private String placeId;
	private String colorName;
	private String colorId;
	
	public ExtendedDoFInitialTokenPopulation(int[] values, String placeName, String placeId, String colorName, String colorId) {
		this.values = values;
		this.placeName = placeName;
		this.placeId = placeId;
		this.colorId = colorId;
		this.colorName = colorName;
		this.setIdentifier(getDQLName());
	}

	//TODO PROBES!!!!
	@Override
	public Element toXML() {
        Document document = DocumentHelper.createDocument();
        Element eDof = document.addElement("ExtendedDoFInitialTokenPopulation")
        		.addAttribute("placeName", placeName)
        		.addAttribute("placeId", placeId)
        		.addAttribute("colorName", colorName)
        		.addAttribute("colorId", colorId)
        		.addAttribute("identifier", getIdentifier());
        if (values != null)
        	for (int value: values)
        		eDof.addElement("value").setText("" + value);
		return eDof;
	}
	
	@Override
	public boolean doesExtend(DoF dof) {
		if (dof.getIdentifier().equals(getDQLName()))
			return true;
		return false;
	}

	@Override
	public void setProperties(Properties p) {
		super.setProperties(p);
		if (p != null && p.get("value") != null)
			this.values = parseDoFString((String)p.get("value"));
	}

	private int[] parseDoFString(String s) {
		s = s.replaceAll("(\\[|\\])", "");
		s = s.replaceAll("\\s", "");
		String[] stringPieces = s.split(",");
		int[] ret = new int[stringPieces.length];
		for (int i = 0; i < stringPieces.length; i++) 
			try {
				ret[i] = Integer.parseInt(stringPieces[i]);
			} catch (NumberFormatException e) {
				throw new DQLException("The DoF parameters have to be numbers, but "+ stringPieces[i] 
						+ "is not");
			}
		return ret;
	}
	
	public String toString() {
		return toXML().asXML();
	}
	
	public Element apply(Element net, int valueNumber) {
		Element copy = net.createCopy();
		Element place = NetHelper.getPlaceById(copy, placeId);
		Element colorRef = PlaceHelper.getColorReferenceByColorId(place, colorId);
		colorRef.addAttribute("initial-population", "" + values[valueNumber]);
		return copy;
	}
	
	public String getDQLName() {
		return placeName + "_" + colorName;
	}
	
	public ExtendedDoFInitialTokenPopulation clone() {
		ExtendedDoFInitialTokenPopulation clone = new ExtendedDoFInitialTokenPopulation(values, placeName, placeId, colorName, colorId);
		clone.setIdentifier(getIdentifier());
		clone.setAlias(getAlias());
		clone.setProperties(getProperties());
		return clone;
	}

	@Override
	public int[] getValues() {
		return values;
	}
}
