package extension;

import java.util.LinkedList;
import java.util.List;

import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EcoreFactory;
import org.eclipse.emf.ecore.EcorePackage;
import org.eclipse.emf.ecore.impl.EClassImpl;
import org.eclipse.emf.ecore.impl.EPackageImpl;
import org.eclipse.emf.ecore.util.EObjectContainmentEList;

import tools.descartes.dql.connector.simQPNV2.DQLException;
import tools.descartes.dql.models.mapping.domain.DomainPackage;
import tools.descartes.dql.models.mapping.domain.Entity;
import tools.descartes.dql.models.mapping.domain.Probe;
import tools.descartes.dql.models.mapping.domain.Resource;
import tools.descartes.dql.models.mapping.domain.Service;
import tools.descartes.dql.models.mapping.domain.impl.EntityImpl;

public class ExtendedEntity extends EntityImpl implements Service, Resource {
	private String xmlId;
	private PlaceType placeType;
	private List<String> colorNames;
	
	
	public ExtendedEntity(String xmlId, PlaceType placeType, String identifier, List<String> colorNames) {
		super();
		this.setIdentifier(identifier);
		this.xmlId = xmlId;
		this.placeType = placeType;
	    
		this.probes = new EObjectContainmentEList<Probe>
		(Probe.class, this, DomainPackage.ENTITY__PROBES);
		
		if (colorNames != null) {
			for (int i = 0; i < colorNames.size(); i++)
				for (int j = i + 1; j < colorNames.size(); j++)
					if (colorNames.get(i).equals(colorNames.get(j)))
						throw new DQLException("There are two colors with the name " + colorNames.get(i) + " within the place "
								+ identifier + ". Please rename one of them!");
			this.colorNames = colorNames;
		} else
			this.colorNames = new LinkedList<String>();
	}
	
	public Element toXML() {
        Document document = DocumentHelper.createDocument();
        Element eEn = document.addElement("ExtendedEntity")
        					.addAttribute("identifier", getIdentifier())
        					.addAttribute("xmlId", xmlId)
        					.addAttribute("placeType", placeType.toString());
		Element probes = eEn.addElement("probes");
        for (Probe p: this.getProbes()) {
        	if (p instanceof ExtendedProbe) {
        		ExtendedProbe ep = (ExtendedProbe) p;
				probes.add(ep.toXML());
        	} else {
        		ExtendedDecimalResult edr = (ExtendedDecimalResult) p;
        		probes.add(edr.toXML());
        	}
		}
        return eEn;
	}
	
	public String getXmlId() {
		return xmlId;
	}
	
	public List<String> getColorNames() {
		return colorNames;
	}
	
	public PlaceType getPlaceType() {
		return placeType;
	}
	
	public boolean doesExtend(Entity e) {
		return e.getIdentifier().equals(identifier);
	}
	
	@Override
	public int hashCode() {
		return 0;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		ExtendedEntity other = (ExtendedEntity) obj;
		return other.getXmlId().equals(getXmlId());
	}

	public ExtendedEntity clone() {
		List<String> newColorNames = new LinkedList<String>();
		newColorNames.addAll(colorNames);
		ExtendedEntity ee = new ExtendedEntity(xmlId, placeType, identifier, newColorNames);
		for (Probe p: this.getProbes()) {
			if (p instanceof ExtendedProbe) {
				ExtendedProbe ep = (ExtendedProbe) p;
				ee.getProbes().add(ep.clone());
			} else {
				ExtendedDecimalResult edr = (ExtendedDecimalResult) p;
				ee.getProbes().add(edr.clone());
			}
		}
		return ee;
	}
}
