package extension;

import java.math.BigDecimal;
import java.math.BigInteger;

import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;

import de.tud.cs.qpe.editors.query.model.Metric;
import tools.descartes.dql.models.mapping.domain.DecimalResult;
import tools.descartes.dql.models.mapping.domain.DomainFactory;
import tools.descartes.dql.models.mapping.domain.Probe;
import tools.descartes.dql.models.mapping.domain.impl.ProbeImpl;

public class ExtendedProbe extends ProbeImpl implements Probe, Cloneable {
	private Metric simQPNMetric;
	private int statLevel;
	private String colorName;
	private boolean depository;
	
	public ExtendedProbe(Metric simQPNMetric, int statLevel, String colorName,
			boolean depository) {
		super();
		metricName = "";
		if (depository)
			metricName += "Depository_";
		if (colorName != null)
			metricName += colorName + "_";
		metricName += getDQLNameForMetric(simQPNMetric);
		
		this.setMetricName(metricName);
		this.simQPNMetric = simQPNMetric;
		this.statLevel = statLevel;
		this.colorName = colorName;
		this.depository = depository;
	}
	
	public Element toXML() {
        Document document = DocumentHelper.createDocument();
        Element eEn = document.addElement("ExtendedProbe")
        					.addAttribute("metricName", getMetricName())
        					.addAttribute("statLevel", "" + statLevel)
        					.addAttribute("colorName", colorName)
        					.addAttribute("depository", Boolean.toString(depository));
		return eEn;
	}
	
	public Metric getSimQPNMetric() {
		return simQPNMetric;
	}
	
	public int getStatLevel() {
		return statLevel;
	}
	
	public String getColorName() {
		return colorName;
	}
	
	public boolean isMetricForDepository() {
		return depository;
	}
	
	public boolean doesExtend(Probe p) {
		return p.getMetricName().equals(metricName);
	}
	
	public DecimalResult createResult(BigDecimal value, BigDecimal accuracy) {
		ExtendedDecimalResult result = new ExtendedDecimalResult();
		result.setMetricName(metricName);
		result.setValid(true);
		result.setAccuracy(accuracy);
		result.setValue(value);
		return result;
	}
	
	public String toString() {
		return toXML().asXML();
	}
	
	public ExtendedProbe clone() {
		return new ExtendedProbe(simQPNMetric, statLevel, colorName, depository);
	}
	
	private String getDQLNameForMetric(Metric simqpnMetric) {
		switch (simqpnMetric) {
		case ARRIV_THR_PUT:
			return "ArrivalThroughput";
		case CONFIDENCE_INTERVAL_LENGTH:
			return "ConfidenceIntervalHalfLength";
		case CONFIDENCE_INTERVAL_MEAN:
			return "ConfidenceIntervalMean";
		case DEPT_THR_PUT:
			return "DepartureThroughput";
		case MAX_ST:
			return "MaximumTokenResidenceTime";
		case MAX_TK_POP:
			return "MaximumTokenPopulation";
		case MEAN_ST:
			return "MeanTokenResidenceTime";
		case MEAN_TK_POP:
			return "MeanTokenPopulation";
		case MEAN_TOT_TK_POP:
			return "MeanTotalTokenPopulation";
		case MIN_ST:
			return "MinimumTokenResidenceTime";
		case MIN_TK_POP:
			return "MinimumTokenPopulation";
		case QUEUE_UTIL:
			return "QueueUtilization";
		case QUEUE_UTIL_QP1:
			return "QueueUtilizationDueToThisPlace";
		case ST_DEV_ST:
			return "StandardDeviationTokenResidenceTime";
		case TK_COL_OCP:
			return "TokenColorOccupancy";
		case TK_OCP:
			return "TokenOccupancy";
		case TOT_ARRIV_THR_PUT:
			return "TotalArrivalThroughput";
		case TOT_DEPT_THR_PUT:
			return "TotalDepartureThroughput";
		default:
			return "ERROR";
		}
	}
}
