package tools.descartes.dql.connector.simQPNV2;


import java.util.LinkedList;
import java.util.List;

import org.dom4j.Element;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.util.EcoreUtil;

import de.tud.cs.qpe.editors.query.model.Metric;
import de.tud.cs.qpe.model.NetHelper;
import de.tud.cs.qpe.model.PlaceHelper;
import extension.ExtendedDoFInitialTokenPopulation;
import extension.ExtendedEntity;
import extension.ExtendedEntityMapping;
import extension.ExtendedProbe;
import extension.ExtensionUtil;
import extension.PlaceType;
import tools.descartes.dql.connector.ModelStructureQueryConnector;
import tools.descartes.dql.models.mapping.domain.Entity;
import tools.descartes.dql.models.mapping.domain.Resource;
import tools.descartes.dql.models.mapping.domain.Service;
import tools.descartes.dql.models.mapping.mapping.EntityMapping;

public class ModelStructureQueryConnectorImpl implements ModelStructureQueryConnector {
	
	public String getBundleProperty(String key) {
		return Activator.getContext().getProperty(key);
	}
	
	public EntityMapping requestDoF(EntityMapping request) {
		EntityMapping response = new ExtendedEntityMapping();
		Element net = FileUtil.loadXmlFromFile(request.getModelLocation());
		
		for (Element place: NetHelper.listAllPlaces(net)) {
			for (Element color: NetUtil.getColorsForPlace(place))
				response.getDoF().add(new ExtendedDoFInitialTokenPopulation(null, 
						NetUtil.getNameOfElement(place), NetUtil.getIdOfElement(place),
						NetUtil.getNameOfElement(color), NetUtil.getIdOfElement(color)));
		}
		return response;
	}
	
	public EntityMapping requestEntities(EntityMapping request) {
		//TODO any way to validate a QPN? if so, we can remove the duplicate color check in SimQPNExtendedEntity
		Element net = FileUtil.loadXmlFromFile(request.getModelLocation());
		EntityMapping response = new ExtendedEntityMapping();
		response.setModelLocation(request.getModelLocation());
		
		for (Element place: NetHelper.listAllPlaces(net))
			if (isResourceContainedAlready(response.getResources(), NetUtil.getNameOfElement(place)))
				throw new DQLException("There are two places/queues with the name " + NetUtil.getNameOfElement(place)
						+ ". Please rename them before using DQL.");
			else
				response.getResources().add(new ExtendedEntity(NetUtil.getIdOfElement(place),
						differentiateBetweenQueueingAndOrdinaryPlace(place), NetUtil.getNameOfElement(place), getColors(place)));

		for (Element queue: NetHelper.listQueues(net))
			if (isResourceContainedAlready(response.getResources(), NetUtil.getNameOfElement(queue)))
				throw new DQLException("There are two places/queues with the name " + NetUtil.getNameOfElement(queue)
						+ ". Please rename them before using DQL.");
			else
				response.getResources().add(new ExtendedEntity(NetUtil.getIdOfElement(queue),
						PlaceType.QUEUE, NetUtil.getNameOfElement(queue), null));
		
		for (Element probe: NetHelper.listProbes(net))
			if (isServiceContainedAlready(response.getServices(), NetUtil.getNameOfElement(probe)))
				throw new DQLException("There are two probes with the name " + NetUtil.getNameOfElement(probe)
						+ ". Please rename them QPN before using DQL.");
			else
			response.getServices().add(new ExtendedEntity(NetUtil.getIdOfElement(probe),
					PlaceType.PROBE, NetUtil.getNameOfElement(probe), getColors(probe)));

		return response;
	}

	private PlaceType differentiateBetweenQueueingAndOrdinaryPlace(Element place) {
		if (PlaceHelper.isOrdinaryPlace(place))
			return PlaceType.ORDINARYPLACE;
		if (PlaceHelper.isQueueingPlace(place))
			return PlaceType.QUEUEINGPLACE;
		return null;
	}

	private List<String> getColors(Element place) {
		List<String> colorNames = new LinkedList<String>();
		for (Element color: NetUtil.getColorsForPlace(place))
			colorNames.add(NetUtil.getNameOfElement(color));
		return colorNames;
	}

	//TODO I hate ELists^^
	private boolean isServiceContainedAlready(EList<Service> services,
			String nameOfElement) {
		for (Service s: services)
			if (s.getIdentifier().equals(nameOfElement))
				return true;
		return false;
	}

	private boolean isResourceContainedAlready(EList<Resource> resources,
			String nameOfElement) {
		for (Resource r: resources)
			if (r.getIdentifier().equals(nameOfElement))
				return true;
		return false;
	}

	public EntityMapping requestProbes(EntityMapping request) {
		EntityMapping answer = ExtensionUtil.extendEntitiesOnly(request, this);
		List<Entity> entities = new LinkedList<Entity>();
		entities.addAll(answer.getResources());
		entities.addAll(answer.getServices());
		
		for (Entity e: entities) {
			ExtendedEntity simEn = (ExtendedEntity) e;
			switch (simEn.getPlaceType()) {
			case ORDINARYPLACE:
				e.getProbes().addAll(getProbesForPlace(simEn.getColorNames()));
				break;
			case QUEUE:
				e.getProbes().addAll(getProbesForQueue());
				break;
			case QUEUEINGPLACE:
				//TODO Talk to jürgen about depositories
				e.getProbes().addAll(getProbesForQueueingPlace(simEn.getColorNames()));
				break;
			case PROBE:
				e.getProbes().addAll(getProbesForProbes(simEn.getColorNames()));
				break;
			}
		}
		System.out.println(answer.toString());
		return answer;
	}

	private List<ExtendedProbe> getProbesForProbes(
			List<String> colorNames) {
		List<ExtendedProbe> probes = new LinkedList<ExtendedProbe>();

		for (String colorName: colorNames) {
			probes.add(new ExtendedProbe(Metric.ARRIV_THR_PUT, 1, colorName, false));
			probes.add(new ExtendedProbe(Metric.DEPT_THR_PUT, 1, colorName, false));

			probes.add(new ExtendedProbe(Metric.TK_COL_OCP, 2, colorName, false));
			probes.add(new ExtendedProbe(Metric.MIN_TK_POP, 2, colorName, false));
			probes.add(new ExtendedProbe(Metric.MAX_TK_POP, 2, colorName, false));
			probes.add(new ExtendedProbe(Metric.MEAN_TK_POP, 2, colorName, false));
			
			probes.add(new ExtendedProbe(Metric.MAX_ST, 3, colorName, false));
			probes.add(new ExtendedProbe(Metric.MEAN_ST, 3, colorName, false));
			probes.add(new ExtendedProbe(Metric.ST_DEV_ST, 3, colorName, false));
			probes.add(new ExtendedProbe(Metric.MIN_ST, 3, colorName, false));
		}
		
		return probes;
	}

	private List<ExtendedProbe> getProbesForQueueingPlace(
			List<String> colorNames) {
		List<ExtendedProbe> probes = new LinkedList<ExtendedProbe>();
		probes.addAll(getProbesForQueueingPlaceDepository(colorNames));
		probes.addAll(getProbesForQueueingPlaceItself(colorNames));
		return probes;
	}
	
	private List<ExtendedProbe> getProbesForQueueingPlaceItself(
			List<String> colorNames) {
		List<ExtendedProbe> probes = new LinkedList<ExtendedProbe>();

		probes.add(new ExtendedProbe(Metric.QUEUE_UTIL_QP1, 2, null, false));
		
		for (String colorName: colorNames) {
			probes.add(new ExtendedProbe(Metric.DEPT_THR_PUT, 2, colorName, false));
			probes.add(new ExtendedProbe(Metric.TK_COL_OCP, 2, colorName, false));
			probes.add(new ExtendedProbe(Metric.MIN_TK_POP, 2, colorName, false));
			probes.add(new ExtendedProbe(Metric.ARRIV_THR_PUT, 2, colorName, false));
			probes.add(new ExtendedProbe(Metric.MAX_TK_POP, 2, colorName, false));
			probes.add(new ExtendedProbe(Metric.MEAN_TK_POP, 2, colorName, false));

			probes.add(new ExtendedProbe(Metric.MAX_ST, 3, colorName, false));
			probes.add(new ExtendedProbe(Metric.MEAN_ST, 3, colorName, false));
			probes.add(new ExtendedProbe(Metric.ST_DEV_ST, 3, colorName, false));
			probes.add(new ExtendedProbe(Metric.MIN_ST, 3, colorName, false));
		}
		
		return probes;
	}
	
	private List<ExtendedProbe> getProbesForQueueingPlaceDepository(
			List<String> colorNames) {
		List<ExtendedProbe> probes = new LinkedList<ExtendedProbe>();

		probes.add(new ExtendedProbe(Metric.TK_OCP, 2, null, true));
		
		for (String colorName: colorNames) {
			probes.add(new ExtendedProbe(Metric.ARRIV_THR_PUT, 1, colorName, true));
			probes.add(new ExtendedProbe(Metric.DEPT_THR_PUT, 1, colorName, true));

			probes.add(new ExtendedProbe(Metric.TK_COL_OCP, 2, colorName, true));
			probes.add(new ExtendedProbe(Metric.MIN_TK_POP, 2, colorName, true));
			probes.add(new ExtendedProbe(Metric.MAX_TK_POP, 2, colorName, true));
			probes.add(new ExtendedProbe(Metric.MEAN_TK_POP, 2, colorName, true));
			
			probes.add(new ExtendedProbe(Metric.MAX_ST, 3, colorName, true));
			probes.add(new ExtendedProbe(Metric.MEAN_ST, 3, colorName, true));
			probes.add(new ExtendedProbe(Metric.ST_DEV_ST, 3, colorName, true));
			probes.add(new ExtendedProbe(Metric.MIN_ST, 3, colorName, true));
		}
		
		return probes;
	}

	private List<ExtendedProbe> getProbesForQueue() {
		List<ExtendedProbe> probes = new LinkedList<ExtendedProbe>();
		
		probes.add(new ExtendedProbe(Metric.TOT_ARRIV_THR_PUT, 1, null, false));
		probes.add(new ExtendedProbe(Metric.TOT_DEPT_THR_PUT, 1, null, false));

		probes.add(new ExtendedProbe(Metric.QUEUE_UTIL, 2, null, false));
		
		probes.add(new ExtendedProbe(Metric.MEAN_ST, 3, null, false));
		
		return probes;
	}

	private List<ExtendedProbe> getProbesForPlace(
			List<String> colorNames) {
		List<ExtendedProbe> probes = new LinkedList<ExtendedProbe>();

		probes.add(new ExtendedProbe(Metric.TK_OCP, 2, null, false));
		
		for (String colorName: colorNames) {
			probes.add(new ExtendedProbe(Metric.ARRIV_THR_PUT, 1, colorName, false));
			probes.add(new ExtendedProbe(Metric.DEPT_THR_PUT, 1, colorName, false));

			probes.add(new ExtendedProbe(Metric.TK_COL_OCP, 2, colorName, false));
			probes.add(new ExtendedProbe(Metric.MIN_TK_POP, 2, colorName, false));
			probes.add(new ExtendedProbe(Metric.MAX_TK_POP, 2, colorName, false));
			probes.add(new ExtendedProbe(Metric.MEAN_TK_POP, 2, colorName, false));
			
			probes.add(new ExtendedProbe(Metric.MAX_ST, 3, colorName, false));
			probes.add(new ExtendedProbe(Metric.MEAN_ST, 3, colorName, false));
			probes.add(new ExtendedProbe(Metric.ST_DEV_ST, 3, colorName, false));
			probes.add(new ExtendedProbe(Metric.MIN_ST, 3, colorName, false));
		}
		
		return probes;
	}

	@Override
	public boolean supportsDoFRequests() {
		return true;
	}

	@Override
	public boolean supportsEntityRequests() {
		return true;
	}

	@Override
	public boolean supportsProbeRequests() {
		return true;
	}
}
